/* A CRC-32 checksum module */

#include <stdio.h>
#include <errno.h>
#include "crc.h"
#include "crc32.h"

void crc_init(unsigned long *crcptr)
{
    *crcptr = 0xFFFFFFFFUL;
}
    
void crc_process(unsigned long *crcptr, const unsigned char *buffer, size_t length)
{
    const unsigned char *ptr = buffer;
    size_t ctr;
    int k;
 
    for (ctr = 0; ctr < length; ctr++, ptr++)
    {
        k = ( (*crcptr) ^ (*ptr) ) & 0x000000FFUL;
        *crcptr = (( (*crcptr) >> 8 ) & 0x00FFFFFFUL ) ^ crc32_table[k];
    }
}

void crc_finalize(unsigned long *crcptr)
{
    *crcptr = ( ~(*crcptr) ) & 0xFFFFFFFFUL; 
}
   
#define BUFSZ 32768

int crc_from_file(unsigned long *crc, FILE *f)
{
    long offset = ftell(f);
    static unsigned char buffer[BUFSZ];
    size_t l;

    if (offset == -1)
        return 0;

    crc_init(crc);

    if (fseek(f, 0, SEEK_SET) == -1)
        return 0;

    errno=0;
    while (((l = fread(buffer, 1, BUFSZ, f)) > 0) && !errno)
    {
        crc_process(crc, buffer, l);
    }

    if (errno)
        return 0;

    crc_finalize(crc);

    if (fseek(f, offset, SEEK_SET) == -1)
        return 0;

    return 1;
}   
